@@ -514,7 +514,8 @@ window_for_event(unsigned int *window_idx, bool *in_tab_bar) {
         Tab *t = global_state.callback_os_window->tabs + global_state.callback_os_window->active_tab;
         for (unsigned int i = 0; i < t->num_windows; i++) {
             if (contains_mouse(t->windows + i) && t->windows[i].render_data.screen) {
-                *window_idx = i; return t->windows + i;
+                *window_idx = i;
+                return t->windows + i;
             }
         }
     }
@@ -529,8 +530,10 @@ closest_window_for_event(unsigned int *window_idx) {
         Tab *t = global_state.callback_os_window->tabs + global_state.callback_os_window->active_tab;
         for (unsigned int i = 0; i < t->num_windows; i++) {
             Window *w = t->windows + i;
-            double d = distance_to_window(w);
-            if (d < closest_distance) { ans = w; closest_distance = d; *window_idx = i; }
+            if (w->visible) {
+                double d = distance_to_window(w);
+                if (d < closest_distance) { ans = w; closest_distance = d; *window_idx = i; }
+            }
         }
     }
     return ans;
@@ -610,7 +613,7 @@ mouse_event(int button, int modifiers, int action) {
 }
 
 void
-scroll_event(double UNUSED xoffset, double yoffset, int flags) {
+scroll_event(double UNUSED xoffset, double yoffset, int flags, int modifiers) {
     bool in_tab_bar;
     static id_type window_for_momentum_scroll = 0;
     static bool main_screen_for_momentum_scroll = false;
@@ -687,7 +690,7 @@ scroll_event(double UNUSED xoffset, double yoffset, int flags) {
         screen_history_scroll(screen, abs(s), upwards);
     } else {
         if (screen->modes.mouse_tracking_mode) {
-            int sz = encode_mouse_event(w, upwards ? GLFW_MOUSE_BUTTON_4 : GLFW_MOUSE_BUTTON_5, PRESS, 0);
+            int sz = encode_mouse_event(w, upwards ? GLFW_MOUSE_BUTTON_4 : GLFW_MOUSE_BUTTON_5, PRESS, modifiers);
             if (sz > 0) {
                 mouse_event_buf[sz] = 0;
                 for (s = abs(s); s > 0; s--) {
